@@ -4,6 +4,7 @@ from warnings import warn
 import CoolProp.CoolProp as CP
 import numpy as np
 from plotly import graph_objects as go
+from . import _RP
 
 from . import Q_
 from .config.fluids import get_name, normalize_mix
@@ -22,20 +23,19 @@ class State(CP.AbstractState):
 
 
     Creates a state from fluid composition and two properties.
-    Properties should be in SI units, **kwargs can be passed
-    to change units.
+    Properties can be floats (SI units are considered) or pint quantities.
 
     Parameters
     ----------
-    p : float
+    p : float, pint.Quantity
         Pressure
-    T : float
+    T : float, pint.Quantity
         Temperature
-    h : float
+    h : float, pint.Quantity
         Enthalpy
-    s : float
+    s : float, pint.Quantity
         Entropy
-    rho : float
+    rho : float, pint.Quantity
         Specific mass
 
     fluid : dict
@@ -50,10 +50,16 @@ class State(CP.AbstractState):
 
     Examples
     --------
+    >>> import ccp
+    >>> Q_ = ccp.Q_
     >>> fluid = {'Oxygen': 0.2096, 'Nitrogen': 0.7812, 'Argon': 0.0092}
-    >>> s = State.define(fluid=fluid, p=101008, T=273, EOS='HEOS')
-    >>> s.rhomass()
-    1.2893965217814896
+    >>> s = ccp.State.define(p=101008, T=273, fluid=fluid)
+    >>> s.rho()
+    <Quantity(1.28939426, 'kilogram / meter ** 3')>
+    >>> # Using pint quantities
+    >>> s = ccp.State.define(fluid=fluid, p=Q_(1, 'atm'), T=Q_(0, 'degC'))
+    >>> s.h()
+    <Quantity(273291.7, 'joule / kilogram')>
     """
 
     def __init__(self, EOS, _fluid):
@@ -94,7 +100,7 @@ class State(CP.AbstractState):
             self.fluid = fluid_dict
         return fluid_dict
 
-    def gas_constant(self):
+    def gas_constant(self, units=None):
         """Gas constant in joule / (mol kelvin).
 
         Returns
@@ -102,9 +108,12 @@ class State(CP.AbstractState):
         gas_constant : pint.Quantity
             Gas constant (joule / (mol kelvin).
         """
-        return Q_(super().gas_constant(), "joule / (mol kelvin)")
+        gas_constant = Q_(super().gas_constant(), "joule / (mol kelvin)")
+        if units:
+            gas_constant = gas_constant.to(units)
+        return gas_constant
 
-    def molar_mass(self):
+    def molar_mass(self, units=None):
         """Molar mass in kg/mol.
 
         Returns
@@ -112,9 +121,12 @@ class State(CP.AbstractState):
         molar_mass : pint.Quantity
             Molar mass (kg/mol).
         """
-        return Q_(super().molar_mass(), "kg/mol")
+        molar_mass = Q_(super().molar_mass(), "kg/mol")
+        if units:
+            molar_mass = molar_mass.to(units)
+        return molar_mass
 
-    def T(self):
+    def T(self, units=None):
         """Temperature in Kelvin.
 
         Returns
@@ -122,9 +134,12 @@ class State(CP.AbstractState):
         T : pint.Quantity
             Temperature (Kelvin).
         """
-        return Q_(super().T(), "kelvin")
+        T = Q_(super().T(), "kelvin")
+        if units:
+            T = T.to(units)
+        return T
 
-    def p(self):
+    def p(self, units=None):
         """Pressure in Pascal.
 
         Returns
@@ -132,9 +147,12 @@ class State(CP.AbstractState):
         p : pint.Quantity
             Pressure (pascal).
         """
-        return Q_(super().p(), "pascal")
+        p = Q_(super().p(), "pascal")
+        if units:
+            p = p.to(units)
+        return p
 
-    def cp(self):
+    def cp(self, units=None):
         """Specific heat at constant pressure joule/(kilogram kelvin).
 
         Returns
@@ -142,9 +160,28 @@ class State(CP.AbstractState):
         cp : pint.Quantity
             Specific heat at constant pressure joule/(kilogram kelvin).
         """
-        return Q_(super().cpmass(), "joule/(kilogram kelvin)")
+        cp = Q_(super().cpmass(), "joule/(kilogram kelvin)")
+        # use REFPROP directly with forced gas condition if cp value does not converge
+        if cp < 0:
+            fluids = self._fluid.replace("&", "*")
+            r = _RP.REFPROPdll(
+                fluids,
+                "PTV",
+                "Cp",
+                _RP.MASS_BASE_SI,
+                0,
+                0,
+                self.p("kPa").m,
+                self.T().m,
+                self.get_mole_fractions(),
+            )
+            cp = Q_(r.Output[0], "joule/(kilogram kelvin)")
 
-    def cv(self):
+        if units:
+            cp = cp.to(units)
+        return cp
+
+    def cv(self, units=None):
         """Specific heat at constant volume joule/(kilogram kelvin).
 
         Returns
@@ -152,9 +189,12 @@ class State(CP.AbstractState):
         cv : pint.Quantity
             Specific heat at constant volume joule/(kilogram kelvin).
         """
-        return Q_(super().cvmass(), "joule/(kilogram kelvin)")
+        cv = Q_(super().cvmass(), "joule/(kilogram kelvin)")
+        if units:
+            cv = cv.to(units)
+        return cv
 
-    def h(self):
+    def h(self, units=None):
         """Specific Enthalpy (joule/kilogram).
 
         Returns
@@ -162,37 +202,103 @@ class State(CP.AbstractState):
         h : pint.Quantity
             Enthalpy (joule/kilogram).
         """
-        return Q_(super().hmass(), "joule/kilogram")
+        h = Q_(super().hmass(), "joule/kilogram")
+        if units:
+            h = h.to(units)
+        return h
 
-    def s(self):
-        """Specific entropy (per unit of mass)."""
-        return Q_(super().smass(), "joule/(kelvin kilogram)")
+    def s(self, units=None):
+        """Specific entropy (per unit of mass).
 
-    def p_critical(self):
-        """Critical Pressure in Pa"""
-        return Q_(super().p_critical(), "Pa")
+        Returns
+        -------
+        s : pint.Quantity
+            Entropy (joule/(kelvin kilogram)).
+        """
+        s = Q_(super().smass(), "joule/(kelvin kilogram)")
+        if units:
+            s = s.to(units)
+        return s
 
-    def T_critical(self):
-        """Critical Temperature in K"""
-        return Q_(super().T_critical(), "K")
+    def p_critical(self, units=None):
+        """Critical Pressure in Pa.
 
-    def rho(self):
-        """Specific mass (kilogram/m**3)."""
-        return Q_(super().rhomass(), "kilogram/m**3")
+        Returns
+        -------
+        p_critical : pint.Quantity
+            Critical pressure (Pa).
+        """
+        p_critical = Q_(super().p_critical(), "Pa")
+        if units:
+            p_critical = p_critical.to(units)
+        return p_critical
 
-    def v(self):
-        """Specific volume (m**3/kilogram)."""
-        return 1 / self.rho()
+    def T_critical(self, units=None):
+        """Critical Temperature in K.
 
-    def z(self):
+        Returns
+        -------
+        T_critical : pint.Quantity
+            Critical temperature (degK).
+        """
+        T_critical = Q_(super().T_critical(), "K")
+        if units:
+            T_critical.to(units)
+        return T_critical
+
+    def rho(self, units=None):
+        """Specific mass (kilogram/m**3).
+
+        Returns
+        -------
+        rho : pint.Quantity
+            Specific mass (kilogram/m**3).
+        """
+        rho = Q_(super().rhomass(), "kilogram/m**3")
+        if units:
+            rho = rho.to(units)
+        return rho
+
+    def v(self, units=None):
+        """Specific volume (m**3/kilogram).
+
+        Returns
+        -------
+        v : pint.Quantity
+            Specific volume (m**3/kilogram).
+        """
+        v = 1 / self.rho()
+        if units:
+            v = (1 / self.rho()).to(units)
+        return v
+
+    def z(self, units=None):
+        """Compressibility (dimensionless).
+
+        Returns
+        -------
+        z : pint.Quantity
+            Compressibility (dimensionless).
+        """
         z = self.p() * self.molar_mass() / (self.rho() * self.gas_constant() * self.T())
         return z.to("dimensionless")
 
-    def speed_sound(self):
-        """ Speed of sound - Eq. 8.1 from P. Nederstigt - Real Gas Thermodynamics"""
-        return Q_(np.sqrt(self.first_partial_deriv(CP.iP, CP.iDmass, CP.iSmass)), "m/s")
+    def speed_sound(self, units=None):
+        """Speed of sound - Eq. 8.1 from P. Nederstigt - Real Gas Thermodynamics.
 
-    def viscosity(self):
+        Returns
+        -------
+        speed_sound : pint.Quantity
+            Speed of sound (m/s).
+        """
+        speed_sound = Q_(
+            np.sqrt(self.first_partial_deriv(CP.iP, CP.iDmass, CP.iSmass)), "m/s"
+        )
+        if units:
+            speed_sound = speed_sound.to(units)
+        return speed_sound
+
+    def viscosity(self, units=None):
         """Viscosity in pascal second.
 
         Returns
@@ -200,9 +306,12 @@ class State(CP.AbstractState):
         viscosity : pint.Quantity
             Viscosity (pascal second)
         """
-        return Q_(super().viscosity(), "pascal second")
+        viscosity = Q_(super().viscosity(), "pascal second")
+        if units:
+            viscosity = viscosity.to(units)
+        return viscosity
 
-    def kinematic_viscosity(self):
+    def kinematic_viscosity(self, units=None):
         """Kinematic viscosity in m²/s.
 
         Returns
@@ -210,17 +319,23 @@ class State(CP.AbstractState):
         kinematic_viscosity : pint.Quantity
             Kinematic viscosity (m²/s)
         """
-        return (self.viscosity() / self.rho()).to("m²/s")
+        kinematic_viscosity = (self.viscosity() / self.rho()).to("m²/s")
+        if units:
+            kinematic_viscosity = kinematic_viscosity.to(units)
+        return kinematic_viscosity
 
-    def dpdv_s(self):
+    def dpdv_s(self, units=None):
         """
         Partial derivative of pressure to spec. volume with const. entropy.
         """
-        return Q_(
+        dpdv_s = Q_(
             -self.rho().magnitude ** 2
             * (self.first_partial_deriv(CP.iP, CP.iDmass, CP.iSmass)),
             "pascal * kg / m**3",
         )
+        if units:
+            dpdv_s = dpdv_s.to(units)
+        return dpdv_s
 
     def _X(self):
         """ Coeficiente de compressibilidade X de Schultz """
@@ -252,14 +367,18 @@ class State(CP.AbstractState):
         """
         return -(self.v() / self.p()) * self.dpdv_s()
 
-    def dTdp_s(self):
+    def dTdp_s(self, units=None):
         """(dT / dp)s
 
         First partial derivative of temperature related to pressure with
         constant entropy."""
-        return Q_(
+        dTdp_s = Q_(
             super().first_partial_deriv(CP.iT, CP.iP, CP.iSmass), "kelvin / pascal"
         )
+        if units:
+            dTdp_s = dTdp_s.to(units)
+
+        return dTdp_s
 
     def kT(self):
         """Isentropic temperature exponent (dimensionless).
@@ -271,6 +390,19 @@ class State(CP.AbstractState):
         """
         return 1 / (1 - (self.p() / self.T()) * self.dTdp_s())
 
+    def conductivity(self, units=None):
+        """Thermal conductivity (W/m/K).
+
+        Returns
+        -------
+        conductivity : pint.Quantity
+            Thermal conductivity (W/m/K).
+        """
+        conductivity = Q_(super().conductivity(), "W/m/degK")
+        if units:
+            conductivity = conductivity.to(units)
+        return conductivity
+
     def __reduce__(self):
         kwargs = dict(p=self.p(), T=self.T(), fluid=self.fluid)
         return self._rebuild, (self.__class__, kwargs)
@@ -295,20 +427,19 @@ class State(CP.AbstractState):
         """Constructor for state.
 
         Creates a state from fluid composition and two properties.
-        Properties should be in SI units, **kwargs can be passed
-        to change units.
+        Properties can be floats (SI units are considered) or pint quantities.
 
         Parameters
         ----------
-        p : float
+        p : float, pint.Quantity
             Pressure
-        T : float
+        T : float, pint.Quantity
             Temperature
-        h : float
+        h : float, pint.Quantity
             Enthalpy
-        s : float
+        s : float, pint.Quantity
             Entropy
-        rho : float
+        rho : float, pint.Quantity
             Specific mass
 
         fluid : dict
@@ -323,10 +454,16 @@ class State(CP.AbstractState):
 
         Examples
         --------
+        >>> import ccp
+        >>> Q_ = ccp.Q_
         >>> fluid = {'Oxygen': 0.2096, 'Nitrogen': 0.7812, 'Argon': 0.0092}
-        >>> s = State.define(fluid=fluid, p=101008, T=273, EOS='HEOS')
-        >>> s.rhomass()
-        1.2893965217814896
+        >>> s = ccp.State.define(p=101008, T=273, fluid=fluid)
+        >>> s.rho()
+        <Quantity(1.28939426, 'kilogram / meter ** 3')>
+        >>> # Using pint quantities
+        >>> s = ccp.State.define(fluid=fluid, p=Q_(1, 'atm'), T=Q_(0, 'degC'))
+        >>> s.h()
+        <Quantity(273291.7, 'joule / kilogram')>
         """
         if fluid is None:
             raise TypeError("A fluid is required. Provide as fluid=dict(...)")
@@ -334,12 +471,6 @@ class State(CP.AbstractState):
         constituents = []
         molar_fractions = []
 
-        if len(fluid) < 2:
-            warn(
-                "Pure fluids are not fully supported and might break things (e.g. plot_phase_envelope"
-                "See https://github.com/CoolProp/CoolProp/issues/1544"
-            )
-
         for k, v in fluid.items():
             k = get_name(k)
             constituents.append(k)
@@ -450,6 +581,12 @@ class State(CP.AbstractState):
         if fig is None:
             fig = go.Figure()
 
+        if len(self.fluid) < 2:
+            warn(
+                "Pure fluids are not fully supported and might break things (e.g. plot_phase_envelope"
+                "See https://github.com/CoolProp/CoolProp/issues/1544"
+            )
+
         self.build_phase_envelope("dummy")
         phase_envelope = self.get_phase_envelope_data()
         T = Q_(np.array(phase_envelope.T), "degK").to(T_units).m
